import java.awt.*;
import java.awt.event.*;

public class Main extends Frame {
    Main() {
        super("MouseListener Example");
        Component c;
        
        setBackground(Color.yellow);

        // Layout components.
        GridBagLayout gbl = new GridBagLayout();
        setLayout(gbl);

        add(c = new ArrowButton(ArrowButton.NORTH));
        setConstraints(gbl, c, 1, 0);
        add(c = new ArrowButton(ArrowButton.WEST));
        setConstraints(gbl, c, 0, 1);
        add(c = new ArrowButton(ArrowButton.SOUTH));
        setConstraints(gbl, c, 1, 2);
        add(c = new ArrowButton(ArrowButton.EAST));
        setConstraints(gbl, c, 2, 1);

        pack();
        show();
    }

    // Setup the gridbag constraints for a component.
    void setConstraints(GridBagLayout gbl, Component comp, int x, int y) {
        GridBagConstraints cbc = new GridBagConstraints();

        cbc.gridx = x;
        cbc.gridy = y;
        cbc.insets = new Insets(2, 2, 2, 2);
        gbl.setConstraints(comp, cbc);
    }

    public static void main(String args[]) {
        new Main();
    }
}

class ArrowButton extends Component implements MouseListener {
    static final int NORTH = 0;
    static final int WEST = 1;
    static final int SOUTH = 2;
    static final int EAST = 3;

    // ne, nw, sw, se
    int[][] corners = {{1, 0}, {0, 0}, {0, 1}, {1, 1}};
    double[][] tips = {{.5, 0}, {0, .5}, {.5, 1}, {1, .5}};
    int width = 30;
    int height = 30;

    // The orientation of the button.  This value
    // is included in the action events this button fires.
    int orient;
    
    // If true and down is not true, the button should appear red.
    boolean highlight;

    // If true and highlight is true, the button should appear blue.
    boolean down;

    Polygon shape = new Polygon();

    ArrowButton(int o) {
        orient = o;

        // Prepare shape.
        int o1 = (o+1)%corners.length;
        int o2 = (o+2)%corners.length;
        int o3 = (o+3)%corners.length;
        int w = width-1;
        int h = height-1;

        shape.addPoint((int)(tips[o][0]*w), (int)(tips[o][1]*h));
        shape.addPoint(corners[o2][0]*w, corners[o2][1]*h);
        shape.addPoint(corners[o3][0]*w, corners[o3][1]*h);

        // Listen for events.
        addMouseListener(this);
    }

    public void paint(Graphics g) {
        if (highlight) {
            if (down) {
                g.setColor(Color.blue);
            } else {
                g.setColor(Color.red);
            }
        } else {
            g.setColor(Color.white);
        }
        g.fillPolygon(shape);
        g.setColor(Color.black);
        g.drawPolygon(shape);
    }

    // Mouse event handlers
    public void mouseClicked(MouseEvent evt) {
        // do nothing
    }
    public void mousePressed(MouseEvent evt) {
        down = true;
        repaint();
    }
    public void mouseReleased(MouseEvent evt) {
        if (down) {
            // Fire an action event.
            processActionEvent(
                new ActionEvent(this, ActionEvent.ACTION_PERFORMED,
                    ""+orient));
        }
        down = false;
        repaint();
    }
    public void mouseEntered(MouseEvent evt) {
        highlight = true;
        repaint();
    }
    public void mouseExited(MouseEvent evt) {
        highlight = false;
        repaint();
    }

    // Returns true if the coordinate is inside the arrow.
    public boolean contains(int x, int y) {
        return shape.contains(x, y);
    }

    public Dimension getMinimumSize() {
        return new Dimension(width, height);
    }
    public Dimension getPreferredSize() {
        return new Dimension(width, height);
    }

    // Methods to support listeners of the action event.
    transient ActionListener actionListener;

    public synchronized void addActionListener(ActionListener l) {
        actionListener = AWTEventMulticaster.add(actionListener, l);
    }
    public synchronized void removeActionListener(ActionListener l) {
        actionListener = AWTEventMulticaster.remove(actionListener, l);
    }

    protected void processActionEvent(ActionEvent evt) {
        if (actionListener != null) {
            actionListener.actionPerformed(evt);
        }
    }
}
