import java.awt.*;
import java.awt.image.*;
import java.net.*;

class Main extends Frame {
    Main(String filename) {
        super("AreaAveragingScaleFilter Example");
        Image image = null;

        if (filename != null) {
            // Retrieve the image.
            image = getToolkit().getImage(filename);

            // Ensure that the image is completely loaded since we
            // need the width and heigth information immediately.
            MediaTracker tracker = new MediaTracker(this);
            try {
                tracker.addImage(image, 0);
                tracker.waitForID(0);
            } catch (Exception e) {
                e.printStackTrace();
            }
        } else {
            // Type an X in the image.
            Font font = new Font("Serif", Font.BOLD, 72);
            FontMetrics fontM = getFontMetrics(font);
    
            addNotify();
            image = createImage(fontM.charWidth('X'), fontM.getHeight());
            Graphics g = image.getGraphics();
            g.setFont(new Font("Serif", Font.BOLD, 72));
            g.drawString("X", 0, fontM.getAscent());
            g.dispose();
        }

        // Layout components.
        setLayout(new GridLayout(1, 0));
        add(new ImageScaler(image, false));
        add(new ImageScaler(image, true));
        setSize(300, 300);
        show();
    }

    static public void main(String[] args) {
        if (args.length == 1) {
            new Main(args[0]);
        } else {
            new Main(null);
        }
    }
}

class ImageScaler extends Canvas {
    Image origImage;
    Image curImage;
    Dimension curSize = new Dimension(0, 0);
    boolean areaAvr;

    ImageScaler(Image image, boolean areaAvr) {
        origImage = image;
        this.areaAvr = areaAvr;
    }

    public void paint(Graphics g) {
        if (!curSize.equals(getSize())) {
            curSize = new Dimension(getSize());

            // The size has change so create a new scaled image.
            if (areaAvr) {
                curImage = createImage(
                    new FilteredImageSource(origImage.getSource(),
                    //new AreaAveragingScaleFilter(curSize.width, 
                    //  curSize.height)));
                    new AreaAveragingScaleFilter(-2, curSize.height)));
            } else { 
                curImage = createImage(
                    new FilteredImageSource(origImage.getSource(),
                    //new ReplicateScaleFilter(curSize.width, curSize.height)));
                    new ReplicateScaleFilter(-10, curSize.height)));
            }
        }
        g.drawImage(curImage, 0, 0, this);
    }
}
