import java.awt.*;
import java.awt.image.*;
import java.awt.event.*;
import java.net.*;
import java.util.*;

class Main extends Frame implements ActionListener {
    ImageCanvas icv;
    TextField textField = new TextField();

    Main(String filename) {
        super("filterIndexColorModel Example");
        try {
            add(icv = new ImageCanvas(getToolkit().getImage(filename)),
                BorderLayout.CENTER);
            add(textField, BorderLayout.SOUTH);
            textField.addActionListener(this);
        } catch (Exception e) {
            e.printStackTrace();
        }
        setSize(50, 100);
        show();
    }

    public void actionPerformed(ActionEvent evt) {
        icv.seeColor(Integer.parseInt(textField.getText()));
	textField.setText("");
    }

    static public void main(String[] args) {
        if (args.length == 1) {
            new Main(args[0]);
        } else {
            System.err.println("usage: java Main <image file>");
        }
    }
}

class ImageCanvas extends Component {
    Image seeColorImage;
    Image image;
    SeeColorFilter imgf = new SeeColorFilter();

    ImageCanvas(Image image) {
        this.image = image;
        processImage();
    }

    void seeColor(int colorIndex) {
        imgf.seeColor(colorIndex);
        processImage();
    }

    public void paint(Graphics g) {
        g.drawImage(seeColorImage, 0, 0, this);
    }

    void processImage() {
        ImageProducer ip = image.getSource();

        ip = new FilteredImageSource(ip, imgf);
        seeColorImage = getToolkit().createImage(ip);
        repaint();
    }
}

class SeeColorFilter extends RGBImageFilter {
    int colorIndex = -1;

    SeeColorFilter() {
        canFilterIndexColorModel = true;
    }

    void seeColor(int colorIndex) {
        this.colorIndex = colorIndex;
    }

    public IndexColorModel filterIndexColorModel(IndexColorModel icm) {
        int mapsize = icm.getMapSize();
        byte r[] = new byte[mapsize];
        byte g[] = new byte[mapsize];
        byte b[] = new byte[mapsize];

        icm.getReds(r);
        icm.getGreens(g);
        icm.getBlues(b);
        if (colorIndex >= 0) {
            r[colorIndex] = (byte)255;
            g[colorIndex] = 0;
            b[colorIndex] = 0;
        }
        return new IndexColorModel(icm.getPixelSize(), mapsize,
                                   r, g, b, icm.getTransparentPixel());
    }

    public int filterRGB(int x, int y, int rgb) {
        return rgb;
    }
}

