import java.awt.*;
import java.awt.image.*;
import java.awt.event.*;

class Main extends Frame implements ActionListener {
    Main(String upFile, String downFile) {
        super("AWTEventMulticaster Example");
        ImageButton b = new ImageButton(getToolkit().getImage(upFile), 
                                        getToolkit().getImage(downFile), 
                                       "Hello");
        add(b, BorderLayout.CENTER); 
        b.addActionListener(this);
        setBackground(Color.lightGray);
        pack();
        show();
    }

    public void actionPerformed(ActionEvent evt) {
        System.out.println(evt.getActionCommand());
    }

    static public void main(String[] args) {
        if (args.length == 2) {
            new Main(args[0], args[1]);
        } else {
            System.err.println("usage: java Main <image1> <image2>");
        }
    }
}

class ImageButton extends Component {
    Image upImage, downImage;
    transient ActionListener actionListener;
    String cmd;
    boolean raised = true;
    int borderWidth = 5;

    ImageButton(Image upImage, Image downImage, String cmd) {
        this.upImage = upImage;
        this.downImage = downImage;
        this.cmd = cmd;

        // Load first image to get size of button.
        MediaTracker tracker = new MediaTracker(this);
        try {
            tracker.addImage(upImage, 0);
            tracker.waitForAll(0);
        } catch (InterruptedException e) {
            e.printStackTrace();
        }
        addMouseListener(new MouseEventHandler());
    }

    public Dimension getPreferredSize() {
        return new Dimension(upImage.getWidth(null)+2*borderWidth, 
                             upImage.getHeight(null)+2*borderWidth);
    }

    public boolean contains(int x, int y) {
        return (x < upImage.getWidth(null)+2*borderWidth &&
                y < upImage.getHeight(null)+2*borderWidth);
    }

    public void paint(Graphics g) {
        Image image = (raised ? upImage : downImage); 
        g.drawImage(image, borderWidth, borderWidth, this);

        // Draw border around image
        int x = 0, y = 0;
        int w = image.getWidth(this)+2*borderWidth;
        int h = image.getHeight(this)+ 2*borderWidth;
        w -= 1;       // draw inside the rectangle.
        h -= 1;       // draw inside the rectangle.
        g.setColor(getBackground());
        for (int i=0; i<borderWidth; i++) {
            g.draw3DRect(x++, y++, w, h, raised);
            w -= 2;
            h -= 2;
        }
    }

    // Action listener management
    public synchronized void addActionListener(ActionListener l) {
        actionListener = AWTEventMulticaster.add(actionListener, l);
    }
    public void removeActionListener(ActionListener l) {
        actionListener = AWTEventMulticaster.remove(actionListener, l);
    }

    protected void processEvent(AWTEvent evt) {
        if (evt instanceof ActionEvent) {
            processActionEvent((ActionEvent)evt);
            return;
        }
        super.processEvent(evt);
    }

    protected void processActionEvent(ActionEvent evt) {
        if (actionListener != null) {
            actionListener.actionPerformed(evt);
        }
    }

    // Mouse handler for presses and releases inside this button
    class MouseEventHandler extends MouseAdapter {
        public void mousePressed(MouseEvent evt) {
            raised = false;
            repaint();
        }
        public void mouseReleased(MouseEvent evt) {
            raised = true;
            repaint();
            ActionEvent aevt = new ActionEvent(ImageButton.this, 
                                               ActionEvent.ACTION_PERFORMED,
                                               cmd);
            processEvent(aevt);
        }
    }
}
