import java.awt.*;
import java.awt.event.*;

class CanvasWithPopup extends Canvas {
    PopupMenu popup;

    CanvasWithPopup(PopupMenu popup) {
        enableEvents(AWTEvent.MOUSE_EVENT_MASK);
        this.popup = popup;
    }

    void addPopup() {
        add(popup);
    }

    void removePopup() {
        remove(popup);
    }

    protected void processMouseEvent(MouseEvent evt) {
        // Display popup only if it has not been removed
        if (popup.getParent() != null && evt.isPopupTrigger()) {
            popup.show(evt.getComponent(), evt.getX(), evt.getY());
        }
        super.processMouseEvent(evt);
    }
}

public class Main extends Frame implements ItemListener, ActionListener {
    Checkbox cb = new Checkbox("Popup", false);
    CanvasWithPopup canvas;

    Main() {
        super("add Example");
        add(cb, BorderLayout.NORTH); 
        cb.addItemListener(this);

        PopupMenu popup = new PopupMenu("Button Control");
        popup.add("item1");
        popup.add("item2");
        popup.addActionListener(this);
        
        canvas = new CanvasWithPopup(popup);
        add(canvas, BorderLayout.CENTER);

        setSize(100, 200);
        show();
    }

    // Handler for checkbox
    public void itemStateChanged(ItemEvent evt) {
        switch (evt.getStateChange()) {
        case ItemEvent.SELECTED:
            canvas.addPopup();
            break;
        case ItemEvent.DESELECTED:
            canvas.removePopup();
            break;
        }
    }
    
    //  Handler for popup menu
    public void actionPerformed(ActionEvent evt) {
        System.out.println(evt.getActionCommand());
    }

    static public void main(String[] args) {
        new Main();
    }
}
