import java.awt.*;
import java.awt.event.*;

class Main extends Frame {
    Main() {
        super("Container.paint Example");

        // The method causes getInsets() to return correct results.
        addNotify();

        // Get the frame's insets.
        Insets insets = getInsets();

        // Add a few buttons.
        for (int i=0; i<4; i++) {
            FakeButton fb = new FakeButton("Push Me");
            add(fb, 0);
            fb.setBounds(insets.left+i*20, insets.top+i*20, 100, 100);
        }

        // Show the frame.
        setLayout(null);
        setSize(300, 300);
        show();
    }

    public void paint(Graphics g) {
        update(g);
    }

    Image bbuf;
    Graphics bbufG;
    public void update(Graphics g) {
        FontMetrics fm = g.getFontMetrics();
        int w = getSize().width;
        int h = getSize().height;

        // Create or enlarge the double-buffer if necessary.
        if (bbuf == null
                || bbuf.getWidth(null) < w
                || bbuf.getHeight(null) < h) {
            bbuf = createImage(w, h);
            bbufG = bbuf.getGraphics();
        }

        // Needed to workaround a bug in Java 1.1.2.
        bbufG.setClip(0, 0, w, h);
    
        // Paint all of the lightweight's children.
        super.paint(bbufG);

        // Show the double-buffer.
        g.drawImage(bbuf, 0, 0, this);
    }

    public static void main(String[] args) {
        new Main();
    }
}

class FakeButton extends Component implements Runnable {
    String label;
    boolean blink = false;
    boolean blinkOn;

    FakeButton(String l) {
        label = l;

        // Listen for mouse clicks.
        addMouseListener(new MouseEventHandler());

        // Start the blinking thread.
        (new Thread(this)).start();
    }

    public void paint(Graphics g) {
        FontMetrics fm = g.getFontMetrics();
        int w = getSize().width;
        int h = getSize().height;

        g.setColor(Color.black);
        g.fillRect(0, 0, w, h);
        
        // Paint component text.
        if (blink && blinkOn) {
            g.setColor(Color.red);
        } else {
            g.setColor(Color.white);
        }

        // Paint label and outline.
        g.drawString(label, (w-fm.stringWidth(label))/2, 
            (h-fm.getHeight())/2 + fm.getAscent());
        g.drawRect(0, 0, w-1, h-1);
    }

    class MouseEventHandler extends MouseAdapter {
        public void mouseEntered(MouseEvent evt) {
            // Start blinking.
            blink = true;
            repaint();
        }
        public void mouseExited(MouseEvent evt) {
            // Stop blinking.
            blink = false;
            repaint();
        }
    }

    public void run() {
        while (true) {
            try {
                blinkOn = !blinkOn;
                if (blink) {
                    repaint();
                }
                Thread.sleep(250);
            } catch (Exception e) {
            }
        }
    }
}

