import java.awt.*;
import java.awt.image.*;

class Main extends Frame {
    Main(String filename) {
        super("getSource Example");
        add(new ImageCanvas(getToolkit().getImage(filename)),  
            BorderLayout.CENTER); 
        setSize(100, 100);
        show();
    }

    static public void main(String[] args) {
        if (args.length == 1) {
            new Main(args[0]);
        } else {
            System.err.println("usage: java Main <image file>");
        }
    }
}

class ImageCanvas extends Component {
    Image oldImage;
    Image image;

    ImageCanvas(Image image) {
        oldImage = image;
        waitForImage(image);
    }

    void waitForImage(Image image) {
        try {
            // Use a media tracker object to wait until all the pixels 
            // have been retrieved.
            MediaTracker tracker = new MediaTracker(this);
            tracker.addImage(image, 0);
            tracker.waitForID(0);
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    Image getSubImage(Image image, Rectangle r) {
        // Create crop image using a CropImageFilter.
        CropImageFilter imgf = new CropImageFilter(
            r.x, r.y, r.width, r.height);
        ImageProducer ip = image.getSource();

        ip = new FilteredImageSource(ip, imgf);
        Image result = getToolkit().createImage(ip);
        waitForImage(result);
        return result;
    }

    public void paint(Graphics g) {
        int w = oldImage.getWidth(this);
        int h = oldImage.getHeight(this);
        if (image == null) {

            image = createImage(w, h);
            Graphics g2 = image.getGraphics();
            g2.drawImage(getSubImage(oldImage, new Rectangle(0, 0, w/2, h)),
                w/2, 0, this);
            g2.drawImage(getSubImage(oldImage, new Rectangle(w/2, 0, w/2, h)),
                0, 0, this);
            g2.dispose();
        }
        g.drawImage(image, (getSize().width-w)/2, (getSize().height-h)/2, this);
    }
}
