import java.awt.*;
import java.awt.event.*;

class Main extends Frame implements ActionListener, ItemListener {
    final static int ITEMS = 10;
    List ltList = new List(ITEMS, true);
    List rtList = new List(0, true);

    Main() {
        super("List Example");
        GridBagLayout gbl = new GridBagLayout();

        setLayout(gbl);
        add(ltList, 0, 0, 1, 5, 1.0, 1.0);
        add(rtList, 2, 0, 1, 5, 1.0, 1.0);
        // Add action and item listeners to list
        ltList.addActionListener(this);
        ltList.addItemListener(this);
        rtList.addActionListener(this);
        rtList.addItemListener(this);

        // Create buttons for adding/removing items from lists
        Button b;
        add(b = new Button(">"), 1, 0, 1, 1, 0, 1.0);
        b.addActionListener(this);
        add(b = new Button(">>"), 1, 1, 1, 1, 0, 1.0);
        b.addActionListener(this);
        add(b = new Button("<"), 1, 2, 1, 1, 0, 1.0);
        b.addActionListener(this);
        add(b = new Button("<<"), 1, 3, 1, 1, 0, 1.0);
        b.addActionListener(this);
        add(b = new Button("!"), 1, 4, 1, 1, 0, 1.0);
        b.addActionListener(this);

        for (int i=0; i<ITEMS; i++) {
           ltList.add("item "+i);
        }
        pack();
        show();
    }

    void add(Component comp,
            int x, int y, int w, int h, double weightx, double weighty) {
        GridBagLayout gbl = (GridBagLayout)getLayout();
        GridBagConstraints c = new GridBagConstraints();

        c.fill = GridBagConstraints.BOTH;
        c.gridx = x;
        c.gridy = y;
        c.gridwidth = w;
        c.gridheight = h;
        c.weightx = weightx;
        c.weighty = weighty;
        add(comp);
        gbl.setConstraints(comp, c);
    }

    void reverseSelections(List l) {
        for (int i=0; i<l.getItemCount(); i++) {
            if (l.isIndexSelected(i)) {
                l.deselect(i);
            } else {
                l.select(i);
            }
        }
    }

    void deselectAll(List l) {
        for (int i=0; i<l.getItemCount(); i++) {
            l.deselect(i);
        }
    }

    void replaceItem(List l, String item) {
        for (int i=0; i<l.getItemCount(); i++) {
            if (l.getItem(i).equals(item)) {
                l.replaceItem(item + "*", i);
            }
        }
    }

    void move(List l1, List l2, boolean all) {
        if (all) {
            for (int i=0; i<l1.getItemCount(); i++) {
                l2.add(l1.getItem(i));
            }
            l1.removeAll();
        } else {
            String[] items = l1.getSelectedItems();
            int[] itemIndexes = l1.getSelectedIndexes();

            deselectAll(l2);
            for (int i=0; i<items.length; i++) {
                l2.add(items[i]);        // add it
                l2.select(l2.getItemCount()-1);// and select it
                if (i == 0) {
                    l2.makeVisible(l2.getItemCount()-1);
                }
            }
            for (int i=itemIndexes.length-1; i>=0; i--) {
                l1.remove(itemIndexes[i]);
            }
        }
    }

    public void actionPerformed(ActionEvent evt) {
        String arg = evt.getActionCommand();
        if (">".equals(arg)) {
            move(ltList, rtList, false);
        } else if (">>".equals(arg)) {
            move(ltList, rtList, true);
        } else if ("<".equals(arg)) {
            move(rtList, ltList, false);
        } else if ("<<".equals(arg)) {
            move(rtList, ltList, true);
        } else if ("!".equals(arg)) {
            if (ltList.getSelectedItems().length > 0) {
                reverseSelections(ltList);
            } else if (rtList.getSelectedItems().length > 0) {
                reverseSelections(rtList);
            }
        } else {
            Object target = evt.getSource();
            if (target == rtList || target == ltList) {
                replaceItem((List)target, arg);
            }
        } 
    }

    public void itemStateChanged(ItemEvent evt) {
        List target = (List)evt.getSource();
        if (target == ltList) {
            deselectAll(rtList);
        } else if (target == rtList) {
            deselectAll(ltList);
        }
    }

    public static void main(String[] args) {
        new Main();
    }
}
