import java.awt.*;
import java.awt.event.*;

class Main extends Frame {
    Main(String filename) {
        super("ScrollPane Example");
        Image image;
        ScrollPane sp = new ScrollPane();

        // Increase the scrollbar unit increments to 2.
        sp.getHAdjustable().setUnitIncrement(2);
        sp.getVAdjustable().setUnitIncrement(2);

        // Add the canvas to the scrollpane.
        sp.add(new ImageComponent(getToolkit().getImage(filename)));

        // Add the scrollpane to the frame and show.
        add(sp, BorderLayout.CENTER);
        setSize(200, 150);
        show();
    }

    public static void main(String args[]) {
        if (args.length != 1) {
            System.err.println("Usage: java Main <image-file>");
            System.exit(1);
        }
        new Main(args[0]);
    }
}

class ImageComponent extends Component {
    Image image;

    ImageComponent(Image image) {
        this.image = image;
        setBackground(Color.lightGray);

        // Set the cursor to be a hand cursor.
        setCursor(Cursor.getPredefinedCursor(Cursor.HAND_CURSOR));

        // Use the media tracker to retrieve the dimensions
        // of the image since we need them immediately.
        try {
            MediaTracker tracker = new MediaTracker(this);
    
            tracker.addImage(image, 0);
            tracker.waitForAll(0);
        } catch (Exception e) {
            e.printStackTrace();
        }

        // Listen for mouse events.
        addMouseListener(new MouseEventHandler());
        addMouseMotionListener(new MouseMotionEventHandler());
    }

    public Dimension getPreferredSize() {
        return new Dimension(image.getWidth(null), image.getHeight(null));
    }

    public void paint(Graphics g) {
        int iw = image.getWidth(this);
        int ih = image.getHeight(this);

        // Only if the dimensions are available do we draw the image.
        if (iw >= 0 && ih >= 0) {
            g.setColor(getBackground());
            g.clearRect(0, 0, getSize().width, getSize().height);

            // Draw the image centered.
            g.drawImage(image, (getSize().width-iw)/2,
                (getSize().height-ih)/2, this);
        }
    }

    Point downPt;
    class MouseEventHandler extends MouseAdapter {
        public void mousePressed(MouseEvent evt) {
            // Record the location of the down click.
            downPt = evt.getPoint();
        }
    }
        
    class MouseMotionEventHandler extends MouseMotionAdapter {
        public void mouseDragged(MouseEvent evt) {
            ScrollPane sp = (ScrollPane)(getParent().getParent());
            Point pt = sp.getScrollPosition();
            Dimension sz = sp.getViewportSize();
    
            // Calculate new position of viewport.
            pt.translate(downPt.x-evt.getX(), downPt.y-evt.getY());

            // Ensure that the new position is in the valid range.
            pt.x = Math.max(0, Math.min(getSize().width-sz.width, pt.x));
            pt.y = Math.max(0, Math.min(getSize().height-sz.height, pt.y));

            // Update the position of the viewport.
            sp.setScrollPosition(pt);
        }
    }
}
