import java.awt.*;
import java.awt.event.*;

class Main extends Frame implements AdjustmentListener {
    Scrollbar sbVer = new Scrollbar(Scrollbar.VERTICAL);
    Scrollbar sbHor = new Scrollbar(Scrollbar.HORIZONTAL);
    MainCanvas cv = new MainCanvas(sbVer, sbHor);

    Main() {
        super("Scrollbar Example");
        add(cv, BorderLayout.CENTER);
        add(sbVer, BorderLayout.EAST);
        add(sbHor, BorderLayout.SOUTH);

        // Add adjustment listener for scrollbars
        sbVer.addAdjustmentListener(this);
        sbHor.addAdjustmentListener(this);

        setSize(200, 200);
        show();
    }

    public void adjustmentValueChanged(AdjustmentEvent evt) {
      Scrollbar sb = (Scrollbar)evt.getSource();
      System.out.println(((sb.getOrientation() == Scrollbar.VERTICAL) ?
                          "ver" : "hor") 
                         + " li=" + sb.getUnitIncrement()
                         + " pi=" + sb.getBlockIncrement()
                         + " max=" + sb.getMaximum()
                         + " vis=" + sb.getVisibleAmount());
      cv.sbChanged();
    }

    static public void main(String[] args) {
        new Main();
    }
}

// Creates a very large square grid.
// Each grid cell is labelled to help with navigation.
class MainCanvas extends Component {
    int gridSize = 1500;
    int gridCells = 10;
    int cellSize = gridSize/gridCells;
    int originX, originY;
    Scrollbar sbVer;
    Scrollbar sbHor;

    MainCanvas(Scrollbar sbVer, Scrollbar sbHor) {
        this.sbVer = sbVer;
        this.sbHor = sbHor;
        setFont(new Font("Courier", Font.BOLD, 36));
    }

    public void sbChanged() {
        originY = -sbVer.getValue();
        originX = -sbHor.getValue();
        repaint();
    }

    // If the size of the component changes, we need to update
    // the scrollbar values.
    public void doLayout() {
        super.doLayout();
        int w = getSize().width;
        int h = getSize().height;

        sbVer.setValues(sbVer.getValue(), h, sbVer.getMinimum(), gridSize);
        sbVer.setBlockIncrement(Math.min(cellSize, h));
        sbVer.setUnitIncrement(Math.min(cellSize, h/5));
        sbHor.setValues(sbHor.getValue(), w, sbHor.getMinimum(), gridSize);
        sbHor.setBlockIncrement(Math.min(cellSize, w));
        sbHor.setUnitIncrement(Math.min(cellSize, w/5));

        // This ensures that if the grid is larger than the viewable
        // area, you will never see space around the grid.
        originX = -Math.max(0, Math.min(gridSize-w, sbHor.getValue()));
        originY = -Math.max(0, Math.min(gridSize-h, sbVer.getValue()));
    }

    public void paint(Graphics g) {
        FontMetrics fm = g.getFontMetrics();

        // Using translate() simplifies the painting code
        // since it doesn't have to worry about origin changes.
        g.translate(originX, originY);
        g.setColor(Color.red);
        for (int i=0; i<gridCells; i++) {
            int c = i * cellSize;
            g.drawLine(c, 0, c, gridSize);
            g.drawLine(0, c, gridSize, c);
        }
        g.drawRect(0, 0, gridSize-1, gridSize-1);
        for (int i=0; i<gridCells; i++) {
            for (int j=0; j<gridCells; j++) {
                String str = "" + i + "," + j;
                int x = i * cellSize;
                int y = j * cellSize;

                g.drawString(str, x+(cellSize-fm.stringWidth(str))/2, 
                    y+(cellSize-fm.getHeight())/2+fm.getAscent());
            }
        }
    }
}
