import java.awt.*;
import java.awt.event.*;
import java.awt.image.*;
import java.beans.*;
import java.net.*;

public class Bean extends Panel {
    Map map = new Map();

    // Constructor
    public Bean() {
        // Initialize the bean with some data for testing purposes.
        Area[] as = {
            new Area(new Rectangle(0, 0, 40, 40), "Area1"),
            new Area(new Rectangle(100, 100, 40, 40), "Area2")};

        map.setAreas(as);
        map.setImage(loadImage("world.gif"));

        add(map, BorderLayout.CENTER);
    }

    // The image accessor methods.
    public Image getImage() {
        return map.image;
    }

    public void setImage(Image newImage) {
        Image oldImage = map.image;

        map.setImage(newImage);
        pceListeners.firePropertyChange("image", oldImage, newImage);
    }

    // The area accessor methods.
    public Area[] getAreas() {
        return map.areas;
    }

    public void setAreas(Area[] newAreas) {
        Area[] oldAreas = map.areas;

        map.setAreas(newAreas);
        pceListeners.firePropertyChange("areas", oldAreas, newAreas);
    }

    // Create the listener list.
    PropertyChangeSupport pceListeners = 
        new PropertyChangeSupport(this);

    // The listener list wrapper methods.
    public synchronized void addPropertyChangeListener(
                                PropertyChangeListener l) {
        pceListeners.addPropertyChangeListener(l);
    }
    
    public synchronized void removePropertyChangeListener(
                                   PropertyChangeListener l) {
        pceListeners.removePropertyChangeListener(l);
    }

    // This method will load an image relative to the location of the 
    // bean class file.  A bean should always use a method like this one 
    // to load resources that are located relative to itself.  Otherwise the 
    // resource will be looked for relative to the location of the bean 
    // container's current working directory.
    public Image loadImage(String imageName) {
        try {
            URL url = getClass().getResource(imageName);
            return getToolkit().createImage((ImageProducer)url.getContent());
        } catch (Exception e) {
            return null;
        }
    }
}
/* 
    manifest.txt file:
        Name: Bean.class
        Java-Bean: True
    jar command:
        jar cfm bean.jar manifest.txt *.class
*/

