import java.awt.*;
import java.awt.event.*;
import java.beans.*;
import java.util.*;
import java.text.*;

class Main extends Frame {
    Main(BeanDescriptor beanDesc) {
        super("FeatureDescriptor Example");
        TextArea ta = new TextArea("", 10, 40, 
            TextArea.SCROLLBARS_VERTICAL_ONLY);

        // Make the text area readonly.
        ta.setEditable(false);

        // Print the attributes that are in FeatureDescriptor
        ta.append("Name: " + beanDesc.getName() + "\n");
        ta.append("Display name: " + beanDesc.getDisplayName() + "\n");
        ta.append("Short Description: " + 
            beanDesc.getShortDescription() + "\n");
        ta.append("Expert: " + beanDesc.isExpert() + "\n");
        ta.append("Hidden: " + beanDesc.isHidden() + "\n\n");

        // Print all the extension attributes.
        Enumeration enum = beanDesc.attributeNames();
        for (Enumeration e=enum; e.hasMoreElements(); ) {
            String name = (String)e.nextElement();
            Object value = beanDesc.getValue(name);

            // Look for Calendar objects.
            if (value instanceof Calendar) {
                value = DateFormat.getDateInstance().format(
                    ((Calendar)value).getTime());
            }
            ta.append(name + " = " + value + "\n");
        }
        
        // Layout and show.
        add(ta, BorderLayout.CENTER);
        setSize(200, 100);
        show();
    }

    public static void main(String[] args) {
        try {
            // Retrieve the bean-info and then the bean descriptor.
            BeanInfo beanInfo = Introspector.getBeanInfo(Bean.class);
            BeanDescriptor beanDesc = beanInfo.getBeanDescriptor();

            // Display the bean descriptor information.
            new Main(beanDesc);
        } catch (IntrospectionException e) {
            e.printStackTrace();
        }
    }
}
