import java.awt.*;
import java.beans.*;

class Main implements PropertyChangeListener {
    Main() {
        Bean bean = null;

        try {
            // Create the bean.
            bean = (Bean)Beans.instantiate(Main.class.getClassLoader(), "Bean");
        } catch (Exception e) {
            e.printStackTrace();
        }
        bean.addPropertyChangeListener(this);
    }

    public void propertyChange(PropertyChangeEvent evt) {
        float p1 = ((Float)evt.getOldValue()).floatValue();
        float p2 = ((Float)evt.getNewValue()).floatValue();

        // Only show two digits after decimal point.
        float diff = Math.round((p2-p1)*100)/100.0f;

        System.out.print(evt.getPropertyName());
        if (p2 > p1) {
            System.out.println(" increase by " + diff);
        } else {
            System.out.println(" decrease by " + -diff);
        }
    }

    public static void main(String args[]) {
        new Main();
    }
}

public class Bean implements Runnable {
    float price = 1.49f;

    // Constructor
    public Bean() {
        (new Thread(this)).start();
    }

    // The price is a bound property.
    public float getPrice() {
        return price;
    }

    public void setPrice(float newPrice) {
        float oldPrice = price;

        price = newPrice;
        pceListeners.firePropertyChange("price", 
            new Float(oldPrice), new Float(newPrice));
    }

    // Create the listener list.
    PropertyChangeSupport pceListeners = 
        new PropertyChangeSupport(this);

    // The listener list wrapper methods.
    public synchronized void addPropertyChangeListener(
                                PropertyChangeListener l) {
        pceListeners.addPropertyChangeListener(l);

    }
    // removePropertyChangeListener() method omitted...

    public void run() {
        try {
            while (true) {
                // Change price up or down by .50.
                setPrice(Math.max(1.0f, price + .5f - (float)Math.random()));

                // Sleep for a random amount of time.
                Thread.sleep(Math.round(Math.random() * 5000));
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
    }
}
